import type { SerializedEditorState } from '@payloadcms/richtext-lexical/lexical'

import config from '@/payload.config'
import { format } from 'date-fns'
import Image from 'next/image'
import { notFound } from 'next/navigation'
import { getPayload } from 'payload'
import React from 'react'
import { RichTextRenderer } from '../../components/RichTextRenderer'

interface NewsArticle {
  id: string
  headline: string
  subheadline?: string
  slug: string
  featuredImage?: {
    url: string
    width: number
    height: number
    alt: string
  }
  content: SerializedEditorState
  excerpt?: string
  publishedDate: string
  tag: {
    id: string
    name: string
  }
  author: {
    id: string
    email: string
  }
}

export default async function NewsArticlePage({
  params,
}: {
  params: { slug: string }
}) {
  // await prevents warning in browser console
  const { slug } = await params

  const payload = await getPayload({ config })

  const result = await payload.find({
    collection: 'news-articles',
    where: {
      slug: {
        equals: slug,
      },
    },
    depth: 5,
  })

  if (!result.docs || result.docs.length === 0) {
    notFound()
  }

  const article = result.docs[0] as unknown as NewsArticle

  return (
    <main className="container mx-auto py-8 px-4">
      <article className="max-w-4xl mx-auto">
        <div className="mb-8 mt-8 md:mt-0">
          <h1 className="text-4xl font-bold mb-2">{article.headline}</h1>
          {article.subheadline && (
            <h2 className="text-xl text-gray-600 mb-4">{article.subheadline}</h2>
          )}

          <div className="flex items-center text-sm text-gray-500 mb-6">
            <span>
              By
              {` ${article.author.email}`}
            </span>
            <span className="mx-2">•</span>
            <time dateTime={article.publishedDate}>
              {format(new Date(article.publishedDate), 'MMMM d, yyyy')}
            </time>
            <span className="mx-2">•</span>
            <span>{article.tag.name}</span>
          </div>
        </div>

        {article.featuredImage && (
          <div className="mb-8 relative rounded-lg overflow-hidden">
            <Image
              src={article.featuredImage.url}
              alt={article.featuredImage.alt || article.headline}
              width={article.featuredImage.width}
              height={article.featuredImage.height}
              className="w-full h-auto"
            />
          </div>
        )}

        <div className="prose prose-lg max-w-none">
          <RichTextRenderer content={article.content} />
        </div>
      </article>
    </main>
  )
}
