import type { Publication } from '../types'
import {
  Accordion,
  AccordionContent,
  AccordionItem,
  AccordionTrigger,
} from '@/components/ui/accordion'
import { Badge } from '@/components/ui/badge'
import { Button } from '@/components/ui/button'
import { Card, CardContent, CardDescription, CardHeader, CardTitle } from '@/components/ui/card'
import CopyButton from '@/components/ui/copy-button'
import { Separator } from '@/components/ui/separator'
import { generateBibTeX } from '@/lib/bibtexGenerator'
import { fetchPublicationById } from '@/lib/publications'
import { ArrowLeft, ExternalLink, FileText } from 'lucide-react'
import Link from 'next/link'
import { notFound } from 'next/navigation'

interface PublicationDetailPageProps {
  params: {
    id: string
  }
}

interface AccessProps {
  publication: Publication
}

export default async function PublicationDetailPage({ params }: PublicationDetailPageProps) {
  const param = await params
  const publication = await fetchPublicationById(param.id)
  const bibTeX = generateBibTeX(publication)

  if (!publication) {
    notFound()
  }

  const authorsList = publication.authors.split(', ')
  const displayedAuthors = authorsList.slice(0, 3).join(', ')
  const hasMoreAuthors = authorsList.length > 3

  return (
    <div className="container max-w-4xl mx-auto py-8 px-4">
      <div className="mb-6 mt-8 md:mt-0">
        <Link href="/publications">
          <Button variant="ghost" className="gap-1 pl-0 hover:pl-1 transition-all">
            <ArrowLeft className="h-4 w-4" />
            Back to Publications
          </Button>
        </Link>
      </div>

      <Card className="border border-gray-200 dark:border-neutral-800 shadow-md bg-white dark:bg-neutral-900 ">
        <CardHeader className="bg-stone-200 dark:bg-stone-800 rounded-t-xl">
          <div className="flex flex-col space-y-2">
            <Badge className="w-fit">{publication.journal}</Badge>
            <CardTitle className="text-2xl font-bold leading-tight">
              {publication.title}
            </CardTitle>

            <div className="text-base text-neutral-600 dark:text-neutral-400">
              {hasMoreAuthors
                ? (
                    <Accordion type="single" collapsible className="w-full">
                      <AccordionItem value="authors" className="border-b-0">
                        <div className="flex items-center">
                          <div>
                            {displayedAuthors}
                            {hasMoreAuthors && ' et al.'}
                          </div>
                          <AccordionTrigger className="py-0 px-2">
                            <span className="sr-only">Show all authors</span>
                          </AccordionTrigger>
                        </div>
                        <AccordionContent>
                          <div className="pt-2 pl-4 border-l-2 border-slate-200 dark:border-slate-700 mt-1">
                            {publication.authors}
                          </div>
                        </AccordionContent>
                      </AccordionItem>
                    </Accordion>
                  )
                : (
                    <div>{publication.authors}</div>
                  )}
            </div>
          </div>
        </CardHeader>

        <CardContent className="pt-6">
          <div className="grid gap-6 md:grid-cols-2">
            <div className="space-y-4">
              <div>
                <h3 className="text-sm font-medium text-slate-500 dark:text-slate-400">Publication Details</h3>
                <Separator className="my-2" />
                <dl className="grid grid-cols-3 gap-1 text-sm">
                  <dt className="font-medium text-slate-700 dark:text-slate-300">Journal:</dt>
                  <dd className="col-span-2">{publication.journal}</dd>

                  <dt className="font-medium text-slate-700 dark:text-slate-300">Year:</dt>
                  <dd className="col-span-2">{publication.published}</dd>

                  {publication.volume && (
                    <>
                      <dt className="font-medium text-slate-700 dark:text-slate-300">Volume:</dt>
                      <dd className="col-span-2">{publication.volume}</dd>
                    </>
                  )}

                  <dt className="font-medium text-slate-700 dark:text-slate-300">Pages:</dt>
                  <dd className="col-span-2">{publication.pages}</dd>

                  <dt className="font-medium text-slate-700 dark:text-slate-300">DOI:</dt>
                  <dd className="col-span-2">
                    <code className="text-xs bg-stone-100 dark:bg-stone-800 px-1 py-0.5 rounded">
                      {publication.doi}
                    </code>
                    <CopyButton name="" textToCopy={publication.doi} />
                  </dd>
                </dl>
              </div>

              {/* Access mobile */}
              <div className="md:hidden">
                <Access publication={publication} />
              </div>

              <div>
                <h3 className="text-sm font-medium text-slate-500 dark:text-slate-400">Abstract</h3>
                <Separator className="my-2" />
                <p className="text-sm text-slate-600 dark:text-slate-300">
                  {publication.abstract || 'This publication explores advancements in the field of quantum computing, specifically focusing on the development of novel algorithms and their applications in solving complex computational problems more efficiently than classical methods.'}
                </p>
              </div>
            </div>

            <div className="space-y-4">
              {/* Access desktop */}
              <div className="md:block hidden">
                <Access publication={publication} />
              </div>

              <div>
                <h3 className="text-sm font-medium text-slate-600 dark:text-slate-400">Citation (BibTeX)</h3>
                <Separator className="my-2" />
                <div className="overflow-x-auto bg-stone-100 dark:bg-stone-800 p-3 rounded-md text-xs font-mono border border-stone-200 dark:border-stone-700">
                  <pre className="whitespace-pre-line md:whitespace-pre">
                    {bibTeX}
                  </pre>
                </div>
                <div className="mt-2 flex justify-end">
                  <CopyButton name="Copy Citation" textToCopy={bibTeX} />
                </div>
              </div>
            </div>
          </div>
        </CardContent>
      </Card>

      <div className="mt-10">
        <div className="flex justify-between items-center mb-4">
          <h2 className="text-lg font-bold">Related Publications</h2>
          <Link href="/publications">
            <Button variant="link" size="sm">
              View All
            </Button>
          </Link>
        </div>

        <div className="grid gap-4 md:grid-cols-2">
          {publication.relatedPublications !== undefined
          && publication.relatedPublications.map(related => (
            <Card key={related.doi} className="overflow-hidden hover:shadow-md transition-all">
              <CardHeader className="p-4">
                <CardTitle className="text-md line-clamp-2">
                  {related.title}
                </CardTitle>
                <CardDescription className="line-clamp-1">
                  {related.authors}
                </CardDescription>
              </CardHeader>
              <CardContent className="p-4 pt-0">
                <div className="flex justify-between text-sm">
                  <div>
                    {related.journal}
                    ,
                    {' '}
                    {related.published}
                  </div>
                  <Link
                    href={`/publications/${related.id}`}
                    rel="noopener noreferrer"
                    className="text-sm text-orange-600 dark:text-orange-400 hover:underline"
                  >
                    View Publication →
                  </Link>
                </div>
              </CardContent>
            </Card>
          ))}
        </div>
      </div>
    </div>
  )
}

function Access({ publication }: AccessProps) {
  return (
    <div>
      <h3 className="text-sm font-medium text-slate-500 dark:text-slate-400">Access</h3>
      <Separator className="my-2" />
      <div className="flex flex-col gap-3">
        <Link
          href={publication.link}
          target="_blank"
          rel="noopener noreferrer"
        >
          <Button className="w-full gap-2 justify-start">
            <ExternalLink className="h-4 w-4" />
            View at Publisher
          </Button>
        </Link>
        {publication.pdfFile?.url
          ? (
              <Link href={publication.pdfFile.url}>
                <Button variant="outline" className="w-full gap-2 justify-start">
                  <FileText className="h-4 w-4" />
                  Download PDF
                </Button>
              </Link>
            )
          : (
              <Button disabled variant="outline" className="w-full gap-2 justify-start">
                <FileText className="h-4 w-4" />
                Download PDF
              </Button>
            )}
      </div>
    </div>
  )
}
